#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for absorber """

import json


def thickness(position, thicknesslist):
    """ code for absorber thickness  datasource

    :param position:  absorber position
    :type position: :obj:`float`
    :param thicknesslist:  thickness JSON list
    :type thicknesslist: :obj:`str`
    :returns: absorber thickness
    :rtype: :obj:`float`
    """
    thicknesslist = json.loads(thicknesslist)
    iposition = int(float(position) + 0.5)
    thickness = []
    for pos, thick in enumerate(thicknesslist):
        thickness.append(thick if (1 << pos) & iposition else 0.)
    return thickness


def foil(position, foillist):
    """ code for absorber foil  datasource

    :param position:  absorber position
    :type position: :obj:`float`
    :param foillist:  foil JSON list
    :type foillist: :obj:`str`
    :returns: absorber foil
    :rtype: :obj:`str`
    """
    foillist = json.loads(foillist)
    iposition = int(float(position) + 0.5)
    foil = []
    for pos, mat in enumerate(foillist):
        foil.append(mat if (1 << pos) & iposition else "")
    return foil



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas.configserver
# \file checks.py
# checks
#
import sys
import xml.etree.ElementTree as et
from lxml.etree import XMLParser
from functools import cmp_to_key


def cmpnodes(n1, n2):
    """ compare etree nodes

    :param n1: first node
    :type n1: :obj:`xml.etree.ElementTree.Element`
    :param n2: second node
    :type n2: :obj:`xml.etree.ElementTree.Element`
    """
    if n1.tag > n2.tag:
        return 1
    elif n1.tag < n2.tag:
        return -1
    elif (n1.tail or "") > (n2.tail or ""):
        return 1
    elif (n1.tail or "") < (n2.tail or ""):
        return -1

    n1at = n1.attrib.items()
    n2at = n2.attrib.items()
    n1at.sort()
    n2at.sort()
    if n1at < n2at:
        return -1
    elif n1at > n2at:
        return 1

    ln1 = list(n1)
    ln2 = list(n2)
    if sys.version_info > (3,):
        ln1.sort(key=cmp_to_key(cmpnodes))
        ln2.sort(key=cmp_to_key(cmpnodes))
    else:
        ln1.sort(cmp=cmpnodes)
        ln2.sort(cmp=cmpnodes)

    for c1, c2 in zip(ln1, ln2):
        val = cmpnodes(c1, c2)
        if val < 0:
            return -1
        elif val > 0:
            return 1
    return 0


def checknodes(utest, n1, n2):
    """ compare etree nodes via unittests

    :param utest: unittest case object
    :type utest: :obj:`unittest.TestCase`
    :param n1: first node
    :type n1: :obj:`xml.etree.ElementTree.Element`
    :param n2: second node
    :type n2: :obj:`xml.etree.ElementTree.Element`
    """
    utest.assertEqual(n1.tag, n2.tag)
    utest.assertEqual(n1.text, n2.text)
    utest.assertEqual(n1.tail, n2.tail)
    utest.assertEqual(len(n1.attrib), len(n2.attrib))
    for k, v in n1.attrib.items():
        utest.assertTrue(k in n2.attrib.keys())
        utest.assertEqual(v, n2.attrib[k])
    utest.assertEqual(len(n1), len(n2))
    ln1 = list(n1)
    ln2 = list(n2)
    if sys.version_info > (3,):
        ln1.sort(key=cmp_to_key(cmpnodes))
        ln2.sort(key=cmp_to_key(cmpnodes))
    else:
        ln1.sort(cmp=cmpnodes)
        ln2.sort(cmp=cmpnodes)
    for c1, c2 in zip(ln1, ln2):
        checknodes(utest, c1, c2)


def checkxmls(utest, xml1, xml2):
    """ compare xmls via unittests

    :param utest: unittest case object
    :type utest: :obj:`unittest.TestCase`
    :param xml1: first xml
    :type xml1: :obj:`str`
    :param xml2: second xml to compare
    :type xml2: :obj:`str`
    """

    n1 = et.fromstring(
        xml1,
        parser=XMLParser(collect_ids=False,
                         remove_blank_text=True))
    n2 = et.fromstring(
        xml2,
        parser=XMLParser(collect_ids=False,
                         remove_blank_text=True))
    try:
        checknodes(utest, n1, n2)
    except Exception:
        print("%s\n!=\n%s" % (xml1, xml2))
        raise


def checknxmls(utest, xml1, xmls):
    """ compare xmls via unittests

    :param utest: unittest case object
    :type utest: :obj:`unittest.TestCase`
    :param xml1: first xml
    :type xml1: :obj:`str`
    :param xmls: list of xml to compare
    :type xmls: :obj:`list` < :obj:`str` >
    """

    n1 = et.fromstring(
        xml1,
        parser=XMLParser(collect_ids=False,
                         remove_blank_text=True))
    ns = []
    for xml2 in xmls:
        ns.append(
            et.fromstring(
                xml2,
                parser=XMLParser(collect_ids=False,
                                 remove_blank_text=True)))
    for i, n2 in enumerate(ns):
        try:
            checknodes(utest, n1, n2)
            break
        except Exception:
            print("%s\n!=\n%s" % (xml1, xml2))
            if i + 1 == len(ns):
                raise



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file XMLConfigurator_test.py
# unittests for field Tags running Tango Server
#
import unittest
# import os
import sys
# import random
# import struct
# import binascii
import time
# import threading
import PyTango
from os.path import expanduser
# import json
# from nxstools import nxscreate

try:
    import NXSCreateClientDSFS_test
except Exception:
    from . import NXSCreateClientDSFS_test

try:
    import ServerSetUp
except ImportError:
    from . import ServerSetUp


if sys.version_info > (3,):
    unicode = str
    long = int


# test fixture
class NXSCreateClientDSDBTest(
        NXSCreateClientDSFS_test.NXSCreateClientDSFSTest):

    # constructor
    # \param methodName name of the test method
    def __init__(self, methodName):
        NXSCreateClientDSFS_test.NXSCreateClientDSFSTest.__init__(
            self, methodName)

        self.__args = '{"host":"localhost", "db":"nxsconfig", ' \
                      '"read_default_file":"/etc/my.cnf", "use_unicode":true}'

        home = expanduser("~")
        self.__args2 = '{"host":"localhost", "db":"nxsconfig", ' \
                       '"read_default_file":"%s/.my.cnf", ' \
                       '"use_unicode":true}' % home
        self._sv = ServerSetUp.ServerSetUp()
        self._proxy = None
        self.flags = " --database --server testp09/testmcs/testr228"

    # opens config server
    # \param args connection arguments
    # \returns NXSConfigServer instance
    def openConfig(self, args):

        found = False
        cnt = 0
        while not found and cnt < 1000:
            try:
                sys.stdout.write(".")
                xmlc = PyTango.DeviceProxy(
                    self._sv.new_device_info_writer.name)
                time.sleep(0.01)
                if xmlc.state() == PyTango.DevState.ON:
                    found = True
                found = True
            except Exception as e:
                print("%s %s" % (self._sv.new_device_info_writer.name, e))
                found = False
            except Exception:
                found = False

            cnt += 1

        if not found:
            raise Exception(
                "Cannot connect to %s"
                % self._sv.new_device_info_writer.name)

        if xmlc.state() == PyTango.DevState.ON:
            xmlc.JSONSettings = args
            xmlc.Open()
        version = xmlc.version
        vv = version.split('.')
        self.revision = long(vv[-1])
        self.version = ".".join(vv[0:3])
        self.label = ".".join(vv[3:-1])

        self.assertEqual(xmlc.state(), PyTango.DevState.OPEN)
        return xmlc

    # closes opens config server
    # \param xmlc XMLConfigurator instance
    def closeConfig(self):
        self.assertEqual(self._proxy.state(), PyTango.DevState.OPEN)

        self._proxy.Close()
        self.assertEqual(self._proxy.state(), PyTango.DevState.ON)

    # test starter
    # \brief Common set up
    def setUp(self):
        NXSCreateClientDSFS_test.NXSCreateClientDSFSTest.setUp(self)
        self._sv.setUp()
        self.openConf()

    # test closer
    # \brief Common tear down
    def tearDown(self):
        NXSCreateClientDSFS_test.NXSCreateClientDSFSTest.tearDown(self)
        self.closeConfig()
        self._sv.tearDown()

    def openConf(self):
        try:
            el = self.openConfig(self.__args)
        except Exception:
            el = self.openConfig(self.__args2)
        self._proxy = el

    def dsexists(self, name):
        avds = self._proxy.availableDataSources()
        return name in avds

    def cpexists(self, name):
        avds = self._proxy.availableComponents()
        return name in avds

    def getds(self, name):
        avds = self._proxy.availableDataSources()
        self.assertTrue(name in avds)
        xmls = self._proxy.datasources([name])
        self.assertEqual(len(xmls), 1)
        return xmls[0]

    def getcp(self, name):
        avcp = self._proxy.availableComponents()
        self.assertTrue(name in avcp)
        xmls = self._proxy.components([name])
        self.assertEqual(len(xmls), 1)
        return xmls[0]

    def deleteds(self, name):
        self._proxy.deleteDataSource(name)

    def deletecp(self, name):
        self._proxy.deleteComponent(name)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for lambdavds """

try:
    import tango
except Exception:
    import PyTango as tango


def triggermode_cb(commonblock, name, triggermode,
                   nbimages, hostname, device,
                   filename, stepindex_str, entryname, insname,
                   eigerdectris_str="EigerDectris",
                   eigerfilewriter_str="EigerFileWriter"):
    """ code for triggermode_cb  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param name: component name
    :type name: :obj:`str`
    :param triggermode:  trigger mode
    :type triggermode: :obj:`int` or :obj:`str`
    :param nbimages: a number of images
    :type nbimages: :obj:`int`
    :param hostname: tango host name
    :type hostname: :obj:`str`
    :param device: tango device name
    :type device: :obj:`str`
    :param filename: file name
    :type filename: :obj:`str`
    :param stepindex_str: name of stepindex datasource
    :type stepindex_str: :obj:`str`
    :param entryname: entry name
    :type entryname: :obj:`str`
    :param insname: instrument name
    :type insname: :obj:`str`
    :param eigerdectris_str: eigerdectris string
    :type eigerdectris_str: :obj:`str`
    :param eigerfilewriter_str: eigerwriter string
    :type eigerfilewriter_str: :obj:`str`
    :returns: triggermode
    :rtype: :obj:`str` or :obj:`int`
    """

    host, port = hostname.split(":")
    port = int(port or 10000)
    edb = tango.Database(host, port)

    sl = edb.get_server_list("%s/*" % (eigerdectris_str))
    writer = None
    for ms in sl:
        devserv = edb.get_device_class_list(ms).value_string
        if device in devserv:
            dev = devserv[0::2]
            serv = devserv[1::2]
            for idx, ser in enumerate(serv):
                if ser == eigerfilewriter_str:
                    writer = dev[idx]
                    break
    wp = tango.DeviceProxy('%s/%s' % (hostname, writer))
    filepattern = wp.FilenamePattern.split("/")[-1]
    imagesperfile = wp.ImagesPerFile
    if filename:
        path = (filename).split("/")[-1].split(".")[0] + "/"
    else:
        path = ""
    path += '%s/%s_' % (name, filepattern)
    totnbimages = sum(commonblock[stepindex_str])
    nbfiles = (totnbimages + imagesperfile - 1) // imagesperfile
    result = triggermode.lower()
    spf = 0
    cfid = 0
    if "__root__" in commonblock.keys():
        root = commonblock["__root__"]
        if hasattr(root, "currentfileid") and hasattr(root, "stepsperfile"):
            spf = root.stepsperfile
            cfid = root.currentfileid
        if root.h5object.__class__.__name__ == "File":
            import nxstools.h5pywriter as nxw
        else:
            import nxstools.h5cppwriter as nxw
    else:
        raise("Writer cannot be found")
    en = root.open(entryname)
    dt = en.open("data")
    ins = en.open(insname)
    det = ins.open(name)
    col = det.open("collection")
    for nbf in range(1, nbfiles+1):
        if spf > 0 and cfid > 0:
            if cfid == nbf:
                nxw.link("%sdata_%06i.h5://entry/data/data" % (path, nbf),
                         det, "data")
                nxw.link("/%s/%s/%s/data" % (entryname, insname, name),
                         dt, name)
            nxw.link("%sdata_%06i.h5://entry/data/data" % (path, nbf),
                     col, "data_%06i" % nbf)
        else:
            nxw.link("%sdata_%06i.h5://entry/data/data" % (path, nbf),
                     col, "data_%06i" % nbf)
            nxw.link("/%s/%s/%s/collection/data_%06i" %
                     (entryname, insname, name, nbf), dt,
                     "%s_%06i" % (name, nbf))
    return result



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file XMLConfigurator_test.py
# unittests for field Tags running Tango Server
#
import unittest
import os
import sys
import random
import struct
import binascii
# import time
# import threading
import PyTango
# import json
from nxstools import nxscreate

try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO

try:
    import TestServerSetUp
except ImportError:
    from . import TestServerSetUp


if sys.version_info > (3,):
    unicode = str
    long = int


# if 64-bit machione
IS64BIT = (struct.calcsize("P") == 8)


# test fixture
class NXSCreatePoolDSFSTest(unittest.TestCase):

    # constructor
    # \param methodName name of the test method
    def __init__(self, methodName):
        unittest.TestCase.__init__(self, methodName)

        try:
            # random seed
            self.seed = long(binascii.hexlify(os.urandom(16)), 16)
        except NotImplementedError:
            import time
            # random seed
            self.seed = long(time.time() * 256)  # use fractional seconds

        self._rnd = random.Random(self.seed)

        self._bint = "int64" if IS64BIT else "int32"
        self._buint = "uint64" if IS64BIT else "uint32"
        self._bfloat = "float64" if IS64BIT else "float32"

        self.__args = '{"host":"localhost", "db":"nxsconfig", ' \
                      '"read_default_file":"/etc/my.cnf", "use_unicode":true}'

        # home = expanduser("~")
        db = PyTango.Database()
        self.host = db.get_db_host().split(".")[0]
        self.port = db.get_db_port()
        self.directory = "."
        self.flags = "-d . "
        self._tsv = TestServerSetUp.TestServerSetUp()

    # sets xmlconfiguration
    # \param xmlc configuration instance
    # \param xml xml configuration string
    def setXML(self, xmlc, xml):
        xmlc.XMLString = xml

    # gets xmlconfiguration
    # \param xmlc configuration instance
    # \returns xml configuration string
    def getXML(self, xmlc):
        return xmlc.XMLString

    # test starter
    # \brief Common set up
    def setUp(self):
        print("\nsetting up...")
        self._tsv.setUp()
        print("SEED = %s" % self.seed)

    # test closer
    # \brief Common tear down
    def tearDown(self):
        self._tsv.tearDown()
        print("tearing down ...")

    def dsexists(self, name):
        return os.path.isfile("%s/%s.ds.xml" % (self.directory, name))

    def cpexists(self, name):
        return os.path.isfile("%s/%s.xml" % (self.directory, name))

    def getds(self, name):
        with open("%s/%s.ds.xml" % (self.directory, name), 'r') as fl:
            xml = fl.read()
        return xml

    def getcp(self, name):
        with open("%s/%s.xml" % (self.directory, name), 'r') as fl:
            xml = fl.read()
        return xml

    def deleteds(self, name):
        os.remove("%s/%s.ds.xml" % (self.directory, name))

    def deletecp(self, name):
        os.remove("%s/%s.xml" % (self.directory, name))

    def runtest(self, argv):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = mystdout = StringIO()
        sys.stderr = mystderr = StringIO()

        old_argv = sys.argv
        sys.argv = argv
        nxscreate.main()
        sys.argv = old_argv

        sys.stdout = old_stdout
        sys.stderr = old_stderr
        vl = mystdout.getvalue()
        er = mystderr.getvalue()
        return vl, er

    def runtestexcept(self, argv, exception):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = mystdout = StringIO()
        sys.stderr = mystderr = StringIO()

        old_argv = sys.argv
        sys.argv = argv
        try:
            error = False
            nxscreate.main()
        except exception:
            error = True
        self.assertEqual(error, True)

        sys.argv = old_argv

        sys.stdout = old_stdout
        sys.stderr = old_stderr
        vl = mystdout.getvalue()
        er = mystderr.getvalue()
        return vl, er

    # Exception tester
    # \param exception expected exception
    # \param method called method
    # \param args list with method arguments
    # \param kwargs dictionary with method arguments
    def myAssertRaise(self, exception, method, *args, **kwargs):
        try:
            error = False
            method(*args, **kwargs)
        except exception:
            error = True
        self.assertEqual(error, True)

    # sets selection configuration
    # \param selectionc configuration instance
    # \param selection selection configuration string
    def setSelection(self, selectionc, selection):
        selectionc.selection = selection

    # gets selectionconfiguration
    # \param selectionc configuration instance
    # \returns selection configuration string
    def getSelection(self, selectionc):
        return selectionc.selection

    def test_poolds_empty(self):
        """ test nxsccreate poolds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        self._tsv.dp.MotorList = [
        ]
        self._tsv.dp.ExpChannelList = [
        ]
        args = [
            [
                ('nxscreate poolds -p %s %s '
                 % (self._tsv.dp.name(), self.flags)).split(),
                [],
                [],
            ],
            [
                ('nxscreate poolds --pool %s %s '
                 % (self._tsv.dp.name(), self.flags)).split(),
                [],
                [],
            ],
        ]

        totest = []
        try:
            for arg in args:
                skip = False
                for ds in arg[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1]:
                        totest.append(ds)

                    vl, er = self.runtest(arg[0])
                    if er:
                        self.assertTrue(er.startswith("Info: "))
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1]):
                        xml = self.getds(ds)
                        self.assertEqual(
                            arg[2][i], xml)

                    for ds in arg[1]:
                        self.deleteds(ds)
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_poolds_pdevice_motors(self):
        """ test nxsccreate poolds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        self._tsv.dp.MotorList = [
            '{'
            '"name": "test_exp_mot_01", '
            '"source" : "tango://haso.desy.de:10000/'
            'motor/omsvme58_exp/10/Position",'
            '"type": "Motor" '
            '}',
            '{'
            '"name": "test_exp_mot_02", '
            '"source" : "tango://haso.desy.de:10000/'
            'motor/omsvme58_exp/20/Position",'
            '"type": "Motor" '
            '}'

        ]
        self._tsv.dp.ExpChannelList = [
        ]
        args = [
            [
                ('nxscreate poolds -p %s %s '
                 % (self._tsv.dp.name(), self.flags)).split(),
                ('nxscreate poolds --pool %s %s '
                 % (self._tsv.dp.name(), self.flags)).split()
            ],
            [
                'test_exp_mot_01',
                'test_exp_mot_02',
            ],
            [
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="test_exp_mot_01">\n'
                '    <device name="motor/omsvme58_exp/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Position"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="test_exp_mot_02">\n'
                '    <device name="motor/omsvme58_exp/20" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Position"/>\n'
                '  </datasource>\n'
                '</definition>\n',
            ],
        ]

        totest = []
        try:
            for cmd in args[0]:
                skip = False
                for ds in args[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in args[1]:
                        totest.append(ds)

                    vl, er = self.runtest(cmd)
                    if er:
                        self.assertTrue(er.startswith("Info: "))
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(args[1]):
                        xml = self.getds(ds)
                        self.assertEqual(
                            args[2][i], xml)

                    for ds in args[1]:
                        self.deleteds(ds)
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_poolds_pdevice_motors_channels(self):
        """ test nxsccreate poolds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        self._tsv.dp.MotorList = [
            '{'
            '"name": "tst_exp_mot01", '
            '"source" : "tango://haso.desy.de:10000/'
            'motor/oms/10/Position",'
            '"type": "Motor" '
            '}',
            '{'
            '"name": "tst_exp_mot02", '
            '"source" : "tango://haso.desy.de:10000/'
            'pmotor/oms/20/Position",'
            '"type": "PseudoMotor" '
            '}'

        ]
        self._tsv.dp.ExpChannelList = [
            '{'
            '"name": "tst_exp_c01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/counter/10/Counts",'
            '"type": "CTExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_adc01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/adc/10/Value",'
            '"type": "ZeroDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_mca01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/mca/10/Value",'
            '"type": "OneDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_det01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/det/10/Data",'
            '"type": "TwoDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_pc01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/pcounter/10/Counts",'
            '"type": "PseudoCounter" '
            '}',
        ]
        args = [
            [
                ('nxscreate poolds -p %s %s '
                 % (self._tsv.dp.name(), self.flags)).split(),
                ('nxscreate poolds --pool %s %s '
                 % (self._tsv.dp.name(), self.flags)).split()
            ],
            [
                'tst_exp_mot01',
                'tst_exp_mot02',
                'tst_exp_c01',
                'tst_exp_adc01',
                'tst_exp_mca01',
                'tst_exp_det01',
                'tst_exp_pc01',
            ],
            [
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mot01">\n'
                '    <device name="motor/oms/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Position"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mot02">\n'
                '    <device name="pmotor/oms/20" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Position"/>\n'
                '  </datasource>\n'
                '</definition>\n',

                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_c01">\n'
                '    <device name="p00/counter/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Counts"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_adc01">\n'
                '    <device name="p00/adc/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Value"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mca01">\n'
                '    <device name="p00/mca/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Value"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_det01">\n'
                '    <device name="p00/det/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Data"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_pc01">\n'
                '    <device name="p00/pcounter/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Counts"/>\n'
                '  </datasource>\n'
                '</definition>\n',
            ],
        ]

        totest = []
        try:
            for cmd in args[0]:
                skip = False
                for ds in args[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in args[1]:
                        totest.append(ds)

                    vl, er = self.runtest(cmd)
                    if er:
                        self.assertTrue(er.startswith("Info: "))
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(args[1]):
                        xml = self.getds(ds)
                        self.assertEqual(
                            args[2][i], xml)

                    for ds in args[1]:
                        self.deleteds(ds)
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_poolds_pdevice_noclientlike(self):
        """ test nxsccreate poolds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        self._tsv.dp.MotorList = [
            '{'
            '"name": "tst_exp_Mot01", '
            '"source" : "tango://haso.desy.de:10000/'
            'motor/oms/10/Position",'
            '"type": "Motor" '
            '}',
            '{'
            '"name": "tst_exp_mot02", '
            '"source" : "tango://haso.desy.de:10000/'
            'pmotor/oms/20/Position",'
            '"type": "PseudoMotor" '
            '}'

        ]
        self._tsv.dp.ExpChannelList = [
            '{'
            '"name": "tst_exp_C01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/counter/10/Counts",'
            '"type": "CTExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_adc01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/adc/10/Value",'
            '"type": "ZeroDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_mca01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/mca/10/Value",'
            '"type": "OneDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_det01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/det/10/Data",'
            '"type": "TwoDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_pc01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/pcounter/10/Counts",'
            '"type": "PseudoCounter" '
            '}',
        ]
        args = [
            [
                ('nxscreate poolds -t -p %s %s '
                 % (self._tsv.dp.name(), self.flags)).split(),
                ('nxscreate poolds --noclientlike --pool %s %s '
                 % (self._tsv.dp.name(), self.flags)).split()
            ],
            [
                'tst_exp_mot01',
                'tst_exp_mot02',
                'tst_exp_c01',
                'tst_exp_adc01',
                'tst_exp_mca01',
                'tst_exp_det01',
                'tst_exp_pc01',
            ],
            [
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mot01">\n'
                '    <device name="motor/oms/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000"/>\n'
                '    <record name="Position"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mot02">\n'
                '    <device name="pmotor/oms/20" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000"/>\n'
                '    <record name="Position"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_c01">\n'
                '    <device name="p00/counter/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000"/>\n'
                '    <record name="Counts"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_adc01">\n'
                '    <device name="p00/adc/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000"/>\n'
                '    <record name="Value"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mca01">\n'
                '    <device name="p00/mca/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000"/>\n'
                '    <record name="Value"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_det01">\n'
                '    <device name="p00/det/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000"/>\n'
                '    <record name="Data"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_pc01">\n'
                '    <device name="p00/pcounter/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000"/>\n'
                '    <record name="Counts"/>\n'
                '  </datasource>\n'
                '</definition>\n',
            ],
        ]

        totest = []
        try:
            for cmd in args[0]:
                skip = False
                for ds in args[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in args[1]:
                        totest.append(ds)

                    vl, er = self.runtest(cmd)
                    if er:
                        self.assertTrue(er.startswith("Info: "))
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(args[1]):
                        xml = self.getds(ds)
                        self.assertEqual(
                            args[2][i], xml)

                    for ds in args[1]:
                        self.deleteds(ds)
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_poolds_pdevice_motors_nolower(self):
        """ test nxsccreate poolds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        self._tsv.dp.MotorList = [
            '{'
            '"name": "tst_exp_mot01", '
            '"source" : "tango://haso.desy.de:10000/'
            'motor/oms/10/Position",'
            '"type": "Motor" '
            '}',
            '{'
            '"name": "tst_exp_Mot02", '
            '"source" : "tango://haso.desy.de:10000/'
            'pmotor/oms/20/Position",'
            '"type": "PseudoMotor" '
            '}'

        ]
        self._tsv.dp.ExpChannelList = [
            '{'
            '"name": "tst_Exp_c01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/counter/10/Counts",'
            '"type": "CTExpChannel" '
            '}',
            '{'
            '"name": "tst_Exp_adc01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/adc/10/Value",'
            '"type": "ZeroDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_mca01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/mca/10/Value",'
            '"type": "OneDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_det01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/det/10/Data",'
            '"type": "TwoDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_pc01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/pcounter/10/Counts",'
            '"type": "PseudoCounter" '
            '}',
        ]
        args = [
            [
                ('nxscreate poolds -n -p %s %s '
                 % (self._tsv.dp.name(), self.flags)).split(),
                ('nxscreate poolds --nolower --pool %s %s '
                 % (self._tsv.dp.name(), self.flags)).split()
            ],
            [
                'tst_exp_mot01',
                'tst_exp_Mot02',
                'tst_Exp_c01',
                'tst_Exp_adc01',
                'tst_exp_mca01',
                'tst_exp_det01',
                'tst_exp_pc01',
            ],
            [
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mot01">\n'
                '    <device name="motor/oms/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Position"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_Mot02">\n'
                '    <device name="pmotor/oms/20" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Position"/>\n'
                '  </datasource>\n'
                '</definition>\n',

                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_Exp_c01">\n'
                '    <device name="p00/counter/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Counts"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_Exp_adc01">\n'
                '    <device name="p00/adc/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Value"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mca01">\n'
                '    <device name="p00/mca/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Value"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_det01">\n'
                '    <device name="p00/det/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Data"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_pc01">\n'
                '    <device name="p00/pcounter/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Counts"/>\n'
                '  </datasource>\n'
                '</definition>\n',
            ],
        ]

        totest = []
        try:
            for cmd in args[0]:
                skip = False
                for ds in args[1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in args[1]:
                        totest.append(ds)

                    vl, er = self.runtest(cmd)
                    if er:
                        self.assertTrue(er.startswith("Info: "))
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(args[1]):
                        xml = self.getds(ds)
                        self.assertEqual(
                            args[2][i], xml)

                    for ds in args[1]:
                        self.deleteds(ds)
        finally:
            for ds in totest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_poolds_pdevice_motors_channels_types(self):
        """ test nxsccreate poolds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        self._tsv.dp.MotorList = [
            '{'
            '"name": "tst_exp_mot01", '
            '"source" : "tango://haso.desy.de:10000/'
            'motor/oms/10/Position",'
            '"type": "Motor" '
            '}',
            '{'
            '"name": "tst_exp_mot02", '
            '"source" : "tango://haso.desy.de:10000/'
            'pmotor/oms/20/Position",'
            '"type": "PseudoMotor" '
            '}'

        ]
        self._tsv.dp.ExpChannelList = [
            '{'
            '"name": "tst_exp_c01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/counter/10/Counts",'
            '"type": "CTExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_adc01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/adc/10/Value",'
            '"type": "ZeroDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_mca01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/mca/10/Value",'
            '"type": "OneDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_det01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/det/10/Data",'
            '"type": "TwoDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_pc01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/pcounter/10/Counts",'
            '"type": "PseudoCounter" '
            '}',
        ]

        types = [
            "Motor",
            "PseudoMotor",
            "CTExpChannel",
            "ZeroDExpChannel",
            "OneDExpChannel",
            "TwoDExpChannel",
            "PseudoCounter"
        ]
        args = [
            [
                ('nxscreate poolds -p %s %s '
                 % (self._tsv.dp.name(), self.flags)).split(),
                ('nxscreate poolds --pool %s %s '
                 % (self._tsv.dp.name(), self.flags)).split()
            ],
            [
                'tst_exp_mot01',
                'tst_exp_mot02',
                'tst_exp_c01',
                'tst_exp_adc01',
                'tst_exp_mca01',
                'tst_exp_det01',
                'tst_exp_pc01',
            ],
            [
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mot01">\n'
                '    <device name="motor/oms/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Position"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mot02">\n'
                '    <device name="pmotor/oms/20" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Position"/>\n'
                '  </datasource>\n'
                '</definition>\n',

                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_c01">\n'
                '    <device name="p00/counter/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Counts"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_adc01">\n'
                '    <device name="p00/adc/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Value"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mca01">\n'
                '    <device name="p00/mca/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Value"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_det01">\n'
                '    <device name="p00/det/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Data"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_pc01">\n'
                '    <device name="p00/pcounter/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Counts"/>\n'
                '  </datasource>\n'
                '</definition>\n',
            ],
        ]

        for tp in types:
            totest = []
            try:
                for cmd in args[0]:
                    skip = False
                    for ds in args[1]:
                        if self.dsexists(ds):
                            skip = True
                    if not skip:
                        for i, ds in enumerate(args[1]):
                            if tp == types[i]:
                                totest.append(ds)

                        ncmd = list(cmd)
                        ncmd.append(tp)
                        vl, er = self.runtest(ncmd)
                        if er:
                            self.assertTrue(er.startswith("Info: "))
                        else:
                            self.assertEqual('', er)
                        self.assertTrue(vl)

                        for i, ds in enumerate(args[1]):
                            if tp == types[i]:
                                xml = self.getds(ds)
                                self.assertEqual(
                                    args[2][i], xml)
                            else:
                                self.myAssertRaise(Exception, self.getds, ds)

                        for i, ds in enumerate(args[1]):
                            if tp == types[i]:
                                self.deleteds(ds)
            finally:
                for ds in totest:
                    if self.dsexists(ds):
                        self.deleteds(ds)

    def test_poolds_pdevice_motors_channels_names(self):
        """ test nxsccreate poolds file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        self._tsv.dp.MotorList = [
            '{'
            '"name": "tst_exp_mot01", '
            '"source" : "tango://haso.desy.de:10000/'
            'motor/oms/10/Position",'
            '"type": "Motor" '
            '}',
            '{'
            '"name": "tst_exp_mot02", '
            '"source" : "tango://haso.desy.de:10000/'
            'pmotor/oms/20/Position",'
            '"type": "PseudoMotor" '
            '}'

        ]
        self._tsv.dp.ExpChannelList = [
            '{'
            '"name": "tst_exp_c01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/counter/10/Counts",'
            '"type": "CTExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_adc01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/adc/10/Value",'
            '"type": "ZeroDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_mca01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/mca/10/Value",'
            '"type": "OneDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_det01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/det/10/Data",'
            '"type": "TwoDExpChannel" '
            '}',
            '{'
            '"name": "tst_exp_pc01", '
            '"source" : "tango://haso.desy.de:10000/'
            'p00/pcounter/10/Counts",'
            '"type": "PseudoCounter" '
            '}',
        ]

        args = [
            [
                ('nxscreate poolds -p %s %s '
                 % (self._tsv.dp.name(), self.flags)).split(),
                ('nxscreate poolds --pool %s %s '
                 % (self._tsv.dp.name(), self.flags)).split()
            ],
            [
                'tst_exp_mot01',
                'tst_exp_mot02',
                'tst_exp_c01',
                'tst_exp_adc01',
                'tst_exp_mca01',
                'tst_exp_det01',
                'tst_exp_pc01',
            ],
            [
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mot01">\n'
                '    <device name="motor/oms/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Position"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mot02">\n'
                '    <device name="pmotor/oms/20" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Position"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_c01">\n'
                '    <device name="p00/counter/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Counts"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_adc01">\n'
                '    <device name="p00/adc/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Value"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_mca01">\n'
                '    <device name="p00/mca/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Value"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_det01">\n'
                '    <device name="p00/det/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Data"/>\n'
                '  </datasource>\n'
                '</definition>\n',
                '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                '<definition>\n'
                '  <datasource type="TANGO" name="tst_exp_pc01">\n'
                '    <device name="p00/pcounter/10" '
                'member="attribute" hostname="haso.desy.de" '
                'port="10000" group="__CLIENT__"/>\n'
                '    <record name="Counts"/>\n'
                '  </datasource>\n'
                '</definition>\n',
            ],
        ]

        for nm in args[1]:
            totest = []
            try:
                for cmd in args[0]:
                    skip = False
                    for ds in args[1]:
                        if self.dsexists(ds):
                            skip = True
                    if not skip:
                        for i, ds in enumerate(args[1]):
                            if nm == args[1][i]:
                                totest.append(ds)

                        ncmd = list(cmd)
                        ncmd.append(nm)
                        vl, er = self.runtest(ncmd)
                        if er:
                            self.assertTrue(er.startswith("Info: "))
                        else:
                            self.assertEqual('', er)
                        self.assertTrue(vl)

                        for i, ds in enumerate(args[1]):
                            if nm == args[1][i]:
                                xml = self.getds(ds)
                                self.assertEqual(
                                    args[2][i], xml)
                            else:
                                self.myAssertRaise(Exception, self.getds, ds)

                        for i, ds in enumerate(args[1]):
                            if nm == args[1][i]:
                                self.deleteds(ds)
            finally:
                for ds in totest:
                    if self.dsexists(ds):
                        self.deleteds(ds)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

""" Filename generator """

import sys


if sys.version_info > (3,):
    unicode = str
    long = int
else:
    bytes = str


class FilenameGenerator(object):

    """Generate image file names

    (c) Copyright 2015 Eugen Wintersberger <eugen.wintersberger@gmail.com>
    (c) Copyright 2015 DESY
    This file is part of nx2img.

    nx2img is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    nx2img is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nx2img.  If not, see <http://www.gnu.org/licenses/>.

    Generator class creating image file names.

    :param fname_template: file name template
    :type fname_template: :obj:`str`
    :param start_index: file start index
    :type start_index: :obj:`int`
    :param stop_index: file stop index
    :type stop_index: :obj:`int`
    """

    def __init__(self, fname_template, start_index=0, stop_index=None):

        #: (:obj:`int`) file start index
        self.file_index = start_index
        #: (:obj:`str`) file name template
        self.file_template = fname_template
        #: (:obj:`int`) file stop index
        self.stop_index = stop_index

    def __call__(self):
        # if self.stop_index is -1 we loop forever, otherwise we stop
        # when the current index exceeds self.stop_index
        while self.stop_index is None or self.file_index <= self.stop_index:
            filename = self.file_template % (self.file_index)
            self.file_index += 1
            yield filename

    @staticmethod
    def from_slice(file_template):
        """
        Static factory method to create a filename_generator instance
        from a sliced user input

        :param file_template: file template
        :type file_template: :obj:`str`
        :returns: filename generator object
        :rtype: :class:`FilenameGenerator`
        """

        (file_format, start_index_str, stop_index_str) = \
            file_template.rsplit(":", 2)
        stop_index = None
        if stop_index_str is not None:
            stop_index = long(stop_index_str)

        start_index = int(start_index_str)

        return FilenameGenerator(file_format, start_index, stop_index)



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for datasignal """

from nxstools import filewriter


def signalname(commonblock, detector, firstchannel,
               timers, mgchannels, entryname, defaultattrs=True):
    """ code for signalname  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param detector: detector name
    :type detector: :obj:`str`
    :param firstchannel: first mg channel
    :type firstchannel: :obj:`str`
    :param timers: a list of timers separated by space
    :type timers: :obj:`str`
    :param mgchannels: a list of mgchannels separated by space
    :type mgchannels: :obj:`str`
    :param entryname:  entry group name
    :type entryname: :obj:`str`
    :param defaultattrs:  set default attributes in NXentry and NXdata
    :type defaultattrs: :obj:`bool`
    :returns: signal name
    :rtype: :obj:`str`
    """

    result = ""
    try:
        timers = [ch for ch in str(timers).split(" ") if ch]
        mgchannels = [ch for ch in str(mgchannels).split(" ") if ch]
        root = commonblock["__root__"]
        if defaultattrs:
            attrs = root.attributes
            at = attrs.create("default", "string", overwrite=True)
            at.write(entryname)
            at.close()
        nxentry = root.open(entryname)
        if defaultattrs:
            attrs = nxentry.attributes
            at = attrs.create("default", "string", overwrite=True)
            at.write("data")
            at.close()
        nxdata = nxentry.open("data")
        writer = root.parent.writer
        links = writer.get_links(nxdata)
        names = list(sorted([ch.name for ch in links]))
        if detector in names:
            result = str(detector)
        elif firstchannel in names:
            result = str(firstchannel)
        elif mgchannels:
            for ch in mgchannels:
                if ch in names and ch not in timers:
                    result = str(ch)
                    break
        if not result:
            for name in names:
                fld = nxdata.open(name)
                if isinstance(fld, filewriter.FTField) and \
                   len(fld.shape) == 1 and fld.shape[0] > 1:
                    result = str(fld.name)
                    break
        if not result and names:
            result = str(names[0])
    except Exception as e:
        result = str(e)
    return result



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for mssar and msnsar """

import pickle
import sys
import json


def mssarenv(msenv, varname):
    """ code for mssar_env  datasource

    :param msenv: sardana env
    :type msenv: :obj:`str`
    :param varname:  env variable name
    :type varname: :obj:`str`
    :returns: sardana variable value
    :rtype: :obj:`any`
    """
    if sys.version_info > (3,):
        msenv = pickle.loads(msenv, encoding='latin1')['new']
    else:
        msenv = pickle.loads(msenv)['new']

    return msenv[varname]


def msnsarenv(msenv, varname):
    """ code for msnsar_env  datasource

    :param msenv: sardana env
    :type msenv: :obj:`str`
    :param varname:  env variable name
    :type varname: :obj:`str`
    :returns: sardana variable value
    :rtype: :obj:`any`
    """
    varname = json.loads(varname)

    if sys.version_info > (3,):
        msenv = pickle.loads(msenv, encoding='latin1')['new']
    else:
        msenv = pickle.loads(msenv)['new']

    result = msenv
    for var in varname:
        result = result[var]
    return result



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for dalsa """


def triggermode(commonblock, name,
                filedir, fileprefix, filepostfix, filestartnum, filesaving,
                triggermode, framespernxfile, pixelformat, height, width,
                acquisitionmode, acquisitionframecount,
                filestartnum_str, nrexposedframes_str,
                filename, entryname, insname):
    """ code for external_data datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param name: detector name
    :type name: :obj:`str`
    :param filedir: file directory
    :type filedir: :obj:`str`
    :param fileprefix: file prefix
    :type fileprefix: :obj:`str`
    :param filepostfix: file postfix
    :type filepostfix: :obj:`str`
    :param filestartnum: file start number
    :type filestartnum: :obj:`int`
    :param filesaving: file saving flag
    :type filesaving: :obj:`int` or :obj:`bool`
    :param triggermode: trigger mode
    :type triggermode: :obj:`str`
    :param framespernxfile: frames per nexus file
    :type framespernxfile: :obj:`int`
    :param pixelformat: pixel format
    :type pixelformat: :obj:`str`
    :param height: image height
    :type height: :obj:`int`
    :param width: image width
    :type width: :obj:`int`
    :param acquisitionmode: acquisition mode
    :type acquisitionmode: :obj:`str`
    :param acquisitionframecount: acquisition frame count
    :type acquisitionframecount: :obj:`str`
    :param filestartnum_str: filestartnum name string
    :type filestartnum_str: :obj:`str`
    :param nrexposedframes_str: nrexposedframes name string
    :type nrexposedframes_str: :obj:`str`
    :param filename: file name
    :type filename: :obj:`str`
    :param entryname: entry name
    :type entryname: :obj:`str`
    :param insname: instrument name
    :type insname: :obj:`str`
    :returns: trigger mode
    :rtype: :obj:`str`
    """
    result = triggermode

    if filesaving:
        if "__root__" in commonblock.keys():
            root = commonblock["__root__"]
        # filenames = []
        filestartnumbers = []
        if filestartnum_str in commonblock:
            filestartnumbers = commonblock[filestartnum_str]
        if nrexposedframes_str in commonblock:
            nrexposedframes = commonblock[nrexposedframes_str]
        flen = len(filestartnumbers)
        frlen = nrexposedframes[-1] if nrexposedframes else 0

        lastfilenumber = filestartnumbers[-1] if filestartnumbers else 0

        totalframenumbers = 0

        mode = acquisitionmode
        framecount = acquisitionframecount
        filesizes = []
        totalframenumbers = []
        nbfiles = 0
        if mode in ["MultiFrame"]:
            totalframenumbers = flen * framecount
            if 1 > framespernxfile or framespernxfile > framecount:
                filesizes = [framecount] * (flen)
                nbfiles = len(filesizes)
                lastfilenbframes = framecount
            else:
                fsizes = []
                filesizes = []
                nbfiles = 0
                nfiles = (framecount + 1) // framespernxfile + 1
                lastfilenbframes = framecount % framespernxfile
                print(nfiles)
                if nfiles:
                    fsizes = [framespernxfile] * (nfiles - 1)
                if lastfilenbframes:
                    fsizes.append(lastfilenbframes)
                for i in range(nfiles):
                    filesizes = filesizes + fsizes
                nbfiles = nfiles * len(fsizes)

        elif mode in ["SingleFrame"]:
            totalframenumbers = flen
            filesizes = [1 for _ in range(flen)]
            nbfiles = len(filesizes)
            lastfilenbframes = nbfiles
        elif mode in ["Continuous"]:
            totalframenumbers = frlen
            if 1 > framespernxfile or framespernxfile > frlen:
                filesizes = [frlen]
                nbfiles = len(filesizes)
                lastfilenbframes = frlen
            else:
                nbfiles = (frlen + 1) // framespernxfile + 1
                lastfilenbframes = frlen % framespernxfile
                filesizes = []
                if nbfiles:
                    filesizes = [framespernxfile] * (nbfiles - 1)
                if lastfilenbframes:
                    filesizes.append(lastfilenbframes)

        dtm = {
            "Mono8": "uint8",
            "Mono8Signed": "int8",
            "Mono10": "uint16",
            "Mono12": "uint16",
            "Mono10Packed": "uint16",
            "Mono12Packed": "uint16",
            "Mono14": "uint16",
            "Mono16": "uint16",
            "Mono16Signed": "int16",
            "BayerGR10": "uint16",
            "BayerRG10": "uint16",
            "BayerGB10": "uint16",
            "BayerBG10": "uint16",
            "BayerGR12": "uint16",
            "BayerRG12": "uint16",
            "BayerGB12": "uint16",
            "BayerBG12": "uint16",
        }

        try:
            dtype = dtm[pixelformat]
        except Exception:
            dtype = "uint8"

        if filename:
            path = filename.split("/")[-1].split(".")[0] + "/"
        else:
            path = ""

        if "__root__" in commonblock.keys():
            root = commonblock["__root__"]
            if root.h5object.__class__.__name__ == "File":
                import nxstools.h5pywriter as nxw
            else:
                import nxstools.h5cppwriter as nxw
        else:
            raise("Writer cannot be found")

        en = root.open(entryname)
        ins = en.open(insname)
        det = ins.open(name)
        npath = "/entry/instrument/detector/data"

        vfl = nxw.virtual_field_layout(
            [totalframenumbers, height, width], dtype)
        firstfilenumber = lastfilenumber - nbfiles
        if nbfiles > 0:
            i1 = 0
            i2 = 0
            for nbf in range(firstfilenumber, lastfilenumber):
                ln = filesizes[nbf - firstfilenumber]
                i1 = i2
                i2 = i2 + ln
                connector = "_%05d." % nbf
                filename = path + name + "/" + str(fileprefix) + connector + \
                    str(filepostfix)
                ef = nxw.target_field_view(
                    filename, npath, [ln, height, width], dtype)
                vfl[i1: i2, :, :] = ef
        det.create_virtual_field("data", vfl)

    return result



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file XMLConfigurator_test.py
# unittests for field Tags running Tango Server
#
import unittest
# import os
import sys
# import random
# import struct
# import binascii
# import time
# import threading
import PyTango
from os.path import expanduser
# import json
# from nxstools import nxscreate

try:
    import NXSCreatePoolDSDB_test
except Exception:
    from . import NXSCreatePoolDSDB_test

try:
    import ServerSetUp
except ImportError:
    from . import ServerSetUp


if sys.version_info > (3,):
    unicode = str
    long = int


# test fixture
class NXSCreatePoolDSDBRTest(
        NXSCreatePoolDSDB_test.NXSCreatePoolDSDBTest):

    # constructor
    # \param methodName name of the test method
    def __init__(self, methodName):
        NXSCreatePoolDSDB_test.NXSCreatePoolDSDBTest.__init__(
            self, methodName)

        self.__args = '{"host":"localhost", "db":"nxsconfig", ' \
                      '"read_default_file":"/etc/my.cnf", "use_unicode":true}'

        home = expanduser("~")
        self.__args2 = '{"host":"localhost", "db":"nxsconfig", ' \
                       '"read_default_file":"%s/.my.cnf", ' \
                       '"use_unicode":true}' % home
        self._sv = ServerSetUp.ServerSetUp()
        self._sv2 = ServerSetUp.ServerSetUp(
            instance="AMCSTEST2",
            dvname="aatestp09/testmcs2/testr228")
        self._proxy = None
        self._proxy2 = None
        self.flags = " --database --server aatestp09/testmcs2/testr228 "

    def openConf2(self):
        try:
            el = self.openConfig(self.__args, self._sv2)
        except Exception:
            el = self.openConfig(self.__args2, self._sv2)
        self._proxy2 = el

    # closes opens config server
    # \param xmlc XMLConfigurator instance
    def closeConfig2(self):
        self.assertEqual(self._proxy2.state(), PyTango.DevState.OPEN)

        self._proxy2.Close()
        self.assertEqual(self._proxy2.state(), PyTango.DevState.ON)

    # test starter
    # \brief Common set up
    def setUp(self):
        self._sv2.setUp()
        self.openConf2()
        NXSCreatePoolDSDB_test.NXSCreatePoolDSDBTest.setUp(self)

    # test closer
    # \brief Common tear down
    def tearDown(self):
        NXSCreatePoolDSDB_test.NXSCreatePoolDSDBTest.tearDown(self)
        self.closeConfig2()
        self._sv2.tearDown()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  pyeval helper functions for pilatus """


def mxparameters_cb(commonblock, mxparameters, name,
                    entryname="scan", insname="instrument"):
    """ code for mxparameters_cb  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param mxparameters:  mx parameters string
    :type mxparameters: :obj:`str`
    :param name: detector name
    :type name: :obj:`str`
    :param entryname: entry name
    :type entryname: :obj:`str`
    :param insname: instrument name
    :type instruement_str: :obj:`str`
    :returns: file start number  - number of frames
    :rtype: :obj:`str`
    """
    pars = mxparameters
    # pars = "# Wavelength 1.03320 A\r\n" \
    #      + "# Detector_distance 0.32200 m\r\n" \
    #      + "# Beam_xy (1261.00, 1242.00) pixels\r\n" \
    #      + "# Filter_transmission 0.1000\r\n" \
    #      + "# Start_angle 204.9240 deg.\r\n" \
    #      + "# Angle_increment 0.1000 deg.\r\n# Phi 404.0470 deg.\r"
    result = pars
    try:
        spars = pars.split("\n")
        tpars = [pr.replace("# ", "").replace("\r", "") for pr in spars]
        tspars = [pr.split(" ") for pr in tpars]
        res = {}
        params = {
            "wavelength": "wavelength",
            "detector_distance": "distance",
            "beam_x": "beam_center_x",
            "beam_y": "beam_center_y",
            "beam_xy": ["beam_center_x", "beam_center_y"]
        }
        for pr in tspars:
            try:
                res[pr[0].lower()] = eval(" ".join(pr[1:]))
            except Exception:
                try:
                    res[pr[0].lower()] = tuple(
                        [eval(" ".join(pr[1:-1])), pr[-1]])
                except Exception:
                    res[pr[0].lower()] = tuple(
                        [" ".join(pr[1:-1]), pr[-1]])
        if res and "__root__" in commonblock.keys():
            root = commonblock["__root__"]
            en = root.open(entryname)
            ins = en.open(insname)
            det = ins.open(name)
            for pname, fname in params.items():
                if pname in res.keys():
                    val = res[pname]
                    if isinstance(val, tuple):
                        val, units = val
                    else:
                        units = ""
                    if not isinstance(val, tuple) and \
                       not isinstance(val, list):
                        fld = det.create_field(fname, "float64")
                        fld.write(val)
                        if units:
                            fld.attributes.create(
                                "units", "string").write(units)
                    elif isinstance(fname, tuple) or isinstance(fname, list):
                        if len(fname) == len(val):
                            for i, fn in enumerate(fname):
                                fld = det.create_field(str(fn), "float64")
                                vl = val[i]
                                fld.write(float(vl))
                                if units:
                                    fld.attributes.create(
                                        "units", "string").write(units)
    except Exception as e:
        # pass
        result += str(e)
    return result


def postrun(commonblock, filestartnum, filedir, nbframes,
            filepostfix, fileprefix,
            filestartnum_str):
    """ code for postrun  datasource

    :param commonblock: commonblock of nxswriter
    :type commonblock: :obj:`dict`<:obj:`str`, `any`>
    :param filestartnum:  file start number
    :type filestartnum: :obj:`int`
    :param filedir: file directorentry name
    :type filedir: :obj:`str`
    :param nbframes: number of frams
    :type nbframes: :obj:`int`
    :param filepostfix: file postfix
    :type filepostfix: :obj:`str`
    :param fileprefix: file prefix
    :type fileprefix: :obj:`str`
    :param filestartnum_str: filestartnum string
    :type filestartnum_str: :obj:`str`
    :returns: postrun string
    :rtype: :obj:`str`
    """
    unixdir = (filedir).replace("\\", "/")
    if len(unixdir) > 1 and unixdir[1] == ":":
        unixdir = "/data" + unixdir[2:]
    if unixdir and unixdir[-1] == "/":
        unixdir = unixdir[:-1]
    filestartnumber = commonblock[filestartnum_str] - 1
    result = "" + unixdir + "/" + fileprefix + "%05d"
    result += filepostfix + ":"
    filelastnumber = filestartnum - 1
    if "__root__" in commonblock.keys():
        root = commonblock["__root__"]
        if hasattr(root, "currentfileid") and hasattr(root, "stepsperfile"):
            spf = root.stepsperfile
            cfid = root.currentfileid
            if spf > 0 and cfid > 0:
                filelastnumber = min(
                    filestartnum + cfid * nbframes * spf - 1, filelastnumber)
                filestartnumber = filestartnumber + (cfid - 1) * nbframes * spf
    result += str(filestartnumber) + ":" + str(filelastnumber)
    return result



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file XMLConfigurator_test.py
# unittests for field Tags running Tango Server
#
import unittest
import sys
import shutil
import os


try:
    import NXSCreateOnlineCPFS_test
except Exception:
    from . import NXSCreateOnlineCPFS_test


if sys.version_info > (3,):
    unicode = str
    long = int


# test fixture
class NXSCreateOnlineCPFS2Test(
        NXSCreateOnlineCPFS_test.NXSCreateOnlineCPFSTest):

    # constructor
    # \param methodName name of the test method
    def __init__(self, methodName):
        NXSCreateOnlineCPFS_test.NXSCreateOnlineCPFSTest.__init__(
            self, methodName)

        self.directory = "my_test_nxs"
        self._dircreated = False
        self.flags = " -d %s " % self.directory
        self.maxDiff = None

    # test starter
    # \brief Common set up
    def setUp(self):
        NXSCreateOnlineCPFS_test.NXSCreateOnlineCPFSTest.setUp(self)
        if not os.path.isdir(self.directory):
            os.makedirs(self.directory)
            self._dircreated = True

    # test closer
    # \brief Common tear down
    def tearDown(self):
        NXSCreateOnlineCPFS_test.NXSCreateOnlineCPFSTest.tearDown(self)
        if self._dircreated:
            shutil.rmtree(self.directory)
            self._dircreated = False

    def test_onlinecp_eigerdectris_fileprefix(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = """<?xml version='1.0' encoding='utf8'?>
<hw>
<device>
 <name>myeigerdectris</name>
 <type>type_tango</type>
 <module>eigerdectris</module>
 <device>p09/eigerdectris/exp.01</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>1</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
</hw>
"""
        args = [
            [
                [
                    ('nxscreate onlinecp -c myeigerdectris -x ts_ '
                     ' %s %s ' % (fname, self.flags)).split(),
                    ('nxscreate onlinecp --file-prefix ts_ '
                     ' --component myeigerdectris '
                     ' %s %s ' % (fname, self.flags)).split(),
                ],
                [
                    ['ts_myeigerdectris'],
                    ['ts_myeigerdectris_autosummationenabled',
                     'ts_myeigerdectris_bitdepth',
                     'ts_myeigerdectris_counttime',
                     'ts_myeigerdectris_description_cb',
                     'ts_myeigerdectris_description',
                     'ts_myeigerdectris_energythreshold',
                     'ts_myeigerdectris_flatfieldenabled',
                     'ts_myeigerdectris_frametime',
                     'ts_myeigerdectris_humidity',
                     'ts_myeigerdectris_nbimages',
                     'ts_myeigerdectris_nbtriggers',
                     'ts_myeigerdectris_photonenergy',
                     'ts_myeigerdectris_ratecorrectionenabled',
                     'ts_myeigerdectris_readouttime',
                     'ts_myeigerdectris_stepindex',
                     'ts_myeigerdectris_temperature',
                     'ts_myeigerdectris_triggermode_cb',
                     'ts_myeigerdectris_triggermode',
                     'ts_myeigerdectris_wavelength'],
                ],
                [
                    ['<?xml version=\'1.0\'?>\n'
                     ''
                     '<definition>\n'
                     '  <group type="NXentry" '
                     'name="$var.entryname#\'scan\'$var.serialno">\n'
                     '    <group type="NXinstrument" name="instrument">\n'
                     '      <group type="NXdetector" name="myeigerdectris">\n'
                     '        <field units="um" type="NX_FLOAT64" '
                     'name="x_pixel_size">75</field>\n'
                     '        <field units="um" type="NX_FLOAT64" '
                     'name="y_pixel_size">75</field>\n'
                     '        <field type="NX_CHAR" name="layout">area'
                     '</field>\n'
                     '        <field type="NX_CHAR" name="description">'
                     '$datasources.myeigerdectris_description_cb'
                     '<strategy mode="INIT"/>\n'
                     '        </field>\n'
                     '        <field type="NX_INT" name="bit_depth_readout">'
                     '$datasources.myeigerdectris_bitdepth<strategy '
                     'mode="FINAL"/>\n'
                     '        </field>\n'
                     '        <field units="s" type="NX_FLOAT64" '
                     'name="detector_readout_time">'
                     '$datasources.myeigerdectris_readouttime'
                     '<strategy mode="FINAL"/>\n'
                     '        </field>\n'
                     '        <field units="s" type="NX_FLOAT64" '
                     'name="count_time">'
                     '$datasources.myeigerdectris_counttime'
                     '<strategy mode="STEP"/>\n'
                     '        </field>\n'
                     '        <field units="s" type="NX_FLOAT64" '
                     'name="frame_time">$datasources.myeigerdectris_frametime'
                     '<strategy mode="STEP"/>\n'
                     '        </field>\n'
                     '        <field units="eV" type="NX_FLOAT64" '
                     'name="threshold_energy">'
                     '$datasources.myeigerdectris_energythreshold'
                     '<strategy mode="FINAL"/>\n'
                     '        </field>\n'
                     '        <field type="NX_BOOLEAN" '
                     'name="flatfield_applied">'
                     '$datasources.myeigerdectris_flatfieldenabled'
                     '<strategy mode="FINAL"/>\n'
                     '        </field>\n'
                     '        <field type="NX_BOOLEAN" '
                     'name="countrate_correction_applied">'
                     '$datasources.myeigerdectris_ratecorrectionenabled'
                     '<strategy mode="FINAL"/>\n'
                     '        </field>\n'
                     '        <group type="NXcollection" name="collection">\n'
                     '          <field type="NX_UINT64" name="nb_images">\n'
                     '            <strategy mode="STEP"/>'
                     '$datasources.myeigerdectris_nbimages</field>\n'
                     '          <field type="NX_UINT64" name="nb_triggers">\n'
                     '            <strategy mode="FINAL"/>'
                     '$datasources.myeigerdectris_nbtriggers</field>\n'
                     '          <field type="NX_CHAR" name="triggermode">\n'
                     '            <strategy mode="FINAL"/>'
                     '$datasources.myeigerdectris_triggermode_cb</field>\n'
                     '          <field type="NX_UINT64" name="stepindex">\n'
                     '            <strategy mode="STEP"/>'
                     '$datasources.myeigerdectris_stepindex</field>\n'
                     '          <field type="NX_BOOLEAN" '
                     'name="auto_summation_applied">'
                     '$datasources.myeigerdectris_autosummationenabled'
                     '<strategy mode="FINAL"/>\n'
                     '          </field>\n'
                     '        </group>\n'
                     '      </group>\n'
                     '    </group>\n'
                     '    <group type="NXdata" name="data">\n'
                     '    </group>\n'
                     '  </group>\n'
                     '</definition>\n'],
                    ['<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_autosummationenabled"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="AutoSummationEnabled"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_bitdepth"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="BitDepth"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_counttime"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="CountTime"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  <datasource type="PYEVAL" '
                     'name="myeigerdectris_description">\n'
                     '    <result name="result">\n'
                     'from nxstools.pyeval import common\n'
                     'common.blockitem_rm(commonblock, '
                     '["myeigerdectris_stepindex""])\n'
                     'ds.result = ds.myeigerdectris_description</result>\n'
                     '  $datasources.myeigerdectris_description\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_description"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="Description"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_energythreshold"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="EnergyThreshold"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_flatfieldenabled"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="FlatFieldEnabled"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_frametime"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="FrameTime"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_humidity"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="Humidity"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_nbimages"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="NbImages"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_nbtriggers"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="NbTriggers"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_photonenergy"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="PhotonEnergy"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO" '
                     'name="myeigerdectris_ratecorrectionenabled"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="RateCorrectionEnabled"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_readouttime"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="ReadoutTime"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  <datasource type="PYEVAL" '
                     'name="myeigerdectris_stepindex">\n'
                     '    <result name="result">\n'
                     'from nxstools.pyeval import common\n'
                     'ds.result = common.blockitem_addint(\n'
                     '    commonblock, "myeigerdectris_stepindex", '
                     'ds.myeigerdectris_nbimages)\n'
                     'ds.result = len(commonblock['
                     '"myeigerdectris_stepindex"])\n'
                     '    </result>\n'
                     '    $datasources.myeigerdectris_nbimages\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_temperature"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="Temperature"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  <datasource type="PYEVAL" '
                     'name="myeigerdectris_triggermode_cb">\n'
                     '    <result name="result">\n'
                     'from nxstools.pyeval import eigerdectris\n'
                     'ds.result = eigerdectris.triggermode_cb(\n'
                     '    commonblock,\n'
                     '    "myeigerdectris",\n'
                     '    ds.myeigerdectris_triggermode,\n'
                     '    ds.myeigerdectris_nbimages,\n'
                     '    "haso000:10000",\n'
                     '    "p09/eigerdectris/exp.01",\n'
                     '    "$var.filename",\n'
                     '    "myeigerdectris_stepindex",\n'
                     '    "$var.entryname#\'scan\'$var.serialno",\n'
                     '    "instrument",\n'
                     '    "EigerDectris",\n'
                     '    "EigerFilewriter"\n'
                     '    )\n'
                     '    </result>\n'
                     '    $datasources.myeigerdectris_triggermode\n'
                     '    $datasources.myeigerdectris_nbimages\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_triggermode"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="TriggerMode"/>\n'
                     '  </datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="TANGO"'
                     ' name="myeigerdectris_wavelength"'
                     '>\n'
                     '    <device name="p09/eigerdectris/exp.01" '
                     'member="attribute" hostname="haso000" '
                     'port="10000" group="myeigerdectris_"/>\n'
                     '    <record name="Wavelength"/>\n'
                     '  </datasource>\n'
                     '</definition>\n'],
                ],
            ],
        ]
        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        with open(fname, "w") as fl:
            fl.write(xml)

        self.checkxmls(args, fname)


if __name__ == '__main__':
    unittest.main()
